local addonName, addonTable = ...

-- Adds to the addonTable:
--  SetMobLevel(level)
--  GetMobLevel()
--  SetDualWielding(state)
--  IsDualWielding()
--  SetUpdateInterval(seconds)
--  StartUpdating()
--  EndUpdating()
--  hitTable
-- Uses from the addonTable:
--  OnHittableUpdated(avoidance)

--------------------------
-- Constants -------------
--------------------------
local SPELL_DODGE = GetSpellInfo(81)
local SPELL_PARRY = GetSpellInfo(3127)
local SPELL_BLOCK = GetSpellInfo(107)

local DW_MISS_PENALTY = 19

--------------------------
-- Variables -------------
--------------------------
local mobLevel = 1
local dualwielding = false
local updateInterval = 1.5

local baseMiss = 5
local k
local class
local baseDefense = 1
local bonusDefense = 0
local critReductionFromTalents = 0
local skillDifference
local chanceFromSkill

local AceEvent = LibStub("AceEvent-3.0")
local AceBucket = LibStub("AceBucket-3.0")
local auraBucket = nil
local wrapper = {}

--------------------------
-- Upvalues --------------
--------------------------
local OnHittableUpdated = addonTable.OnHittableUpdated
local GetDodgeChance, GetParryChance, GetBlockChance = GetDodgeChance, GetParryChance, GetBlockChance
local UnitDefense, UnitClass, IsUsableSpell, GetCombatRatingBonus = UnitDefense, UnitClass, IsUsableSpell, GetCombatRatingBonus

--------------------------
-- Functions -------------
--------------------------
function wrapper.Update()
	-- dummy
end

local UpdateHittable
do
	local hitTable = {}
	addonTable.hitTable = setmetatable({}, {
		__index = hitTable,
		__metatable = false
	})
	local totalChance
	local avoidance
	
	local function SetHittableValue(outcome, probability)
		v = max(0, min(100-totalChance, probability))
		totalChance = totalChance + v
		hitTable[outcome] = v
	end

	UpdateHittable = function()
		totalChance = 0.0

		SetHittableValue("MISS", baseMiss + 1/(0.0625 + k/(0.04*bonusDefense)) - chanceFromSkill)
		SetHittableValue("DODGE", (IsUsableSpell(SPELL_DODGE) and GetDodgeChance()-chanceFromSkill) or 0)
		SetHittableValue("PARRY", (IsUsableSpell(SPELL_PARRY) and GetParryChance()-chanceFromSkill) or 0)
		avoidance = totalChance
		SetHittableValue("BLOCK", (IsUsableSpell(SPELL_BLOCK) and GetBlockChance()-chanceFromSkill) or 0)
		SetHittableValue("CRIT", 5 - 0.04*bonusDefense - GetCombatRatingBonus(CR_CRIT_TAKEN_MELEE) - critReductionFromTalents + chanceFromSkill)
		SetHittableValue("CRUSHING", ((skillDifference > 15) and 2*skillDifference-15) or 0)
		hitTable["HIT"] = 100 - totalChance
		
		OnHittableUpdated(avoidance)
	end
end

function addonTable.SetMobLevel(level)
	if(level ~= mobLevel) then
		skillDifference = level*5 - baseDefense
		chanceFromSkill = 0.04*skillDifference
		mobLevel = level
		wrapper.Update()
	end
end


function addonTable.GetMobLevel()
	return mobLevel
end

function addonTable.SetDualWielding(state)
	if dualwielding ~= state then
		dualwielding = not dualwielding
		if dualwielding then
			baseMiss = baseMiss + DW_MISS_PENALTY
		else
			baseMiss = baseMiss - DW_MISS_PENALTY
		end
		wrapper.Update()
	end
end

function addonTable.IsDualWielding()
	return dualwielding
end

local function OnInventoryOrAurasChanged(unitIds)
	if(unitIds["player"]) then
		_, bonusDefense = UnitDefense("player")
		wrapper.Update()
	end
end

local function OnTalentsChanged()
	wrapper.Update()
end

local function OnBaseDefenseChanged()
	baseDefense = UnitDefense("player")
	skillDifference = mobLevel*5 - baseDefense
	chanceFromSkill = 0.04*skillDifference
	wrapper.Update()
end

function addonTable.SetUpdateInterval(seconds)
	updateInterval = seconds
	if auraBucket then
		AceBucket.UnregisterBucket(addonTable, auraBucket)
		auraBucket = AceBucket.RegisterBucketEvent(addonTable, {"UNIT_AURA", "UNIT_INVENTORY_CHANGED"},
			updateInterval, OnInventoryOrAurasChanged)
		-- Force update, in case unprocessed events were in the bucket we deleted
		wrapper.Update()
	end
end

local function InitializeHittable()
	local CLASS_CONSTANTS = {
		["DEATHKNIGHT"] = 0.956,
		["DRUID"] = 0.972,
		["HUNTER"] = 0.988,
		["MAGE"] = 0.983,
		["PALADIN"] = 0.956,
		["PRIEST"] = 0.983,
		["ROGUE"] = 0.988,
		["SHAMAN"] = 0.988,
		["WARLOCK"] = 0.983,
		["WARRIOR"] = 0.956,
	}
	local CLASS_TALENTFUNCTIONS = {
		["DRUID"] = function() critReductionFromTalents = select(5, GetTalentInfo(2, 18))*2 end
	}

	local _, class = UnitClass("player")
	k = CLASS_CONSTANTS[class]
	local func = CLASS_TALENTFUNCTIONS[class]
	if func then
		OnTalentsChanged = function()
			func()
			wrapper.Update()
		end
	end	
	
	if(select(2, UnitRace("player")) == "NightElf") then
		 baseMiss = baseMiss + 2
	end
end

function addonTable.StartUpdating()
	if InitializeHittable ~= nil then
		InitializeHittable()
		InitializeHittable = nil
	end
	
	OnBaseDefenseChanged()
	OnTalentsChanged()
	OnInventoryOrAurasChanged({["player"] = 1})
	
	UpdateHittable()
	
	-- Swap dummy for real updating function
	UpdateHittable, wrapper.Update = wrapper.Update, UpdateHittable
	
	AceEvent.RegisterEvent(addonTable, "CHARACTER_POINTS_CHANGED", OnTalentsChanged)
	AceEvent.RegisterEvent(addonTable, "PLAYER_TALENT_UPDATE", OnTalentsChanged)
	AceEvent.RegisterEvent(addonTable, "CHAT_MSG_SKILL", OnBaseDefenseChanged)
	auraBucket = AceBucket.RegisterBucketEvent(addonTable, {"UNIT_AURA", "UNIT_INVENTORY_CHANGED"},
			updateInterval, OnInventoryOrAurasChanged)
end

function addonTable.EndUpdating()
	AceEvent.UnregisterEvent(addonTable, "CHARACTER_POINTS_CHANGED")
	AceEvent.UnregisterEvent(addonTable, "PLAYER_TALENT_UPDATE")
	AceEvent.UnregisterEvent(addonTable, "CHAT_MSG_SKILL")
	AceBucket.UnregisterBucket(addonTable, auraBucket)
	auraBucket = nil
	
	-- Swap dummy back in
	UpdateHittable, wrapper.Update = wrapper.Update, UpdateHittable	
end
