local addonName, addonTable = ...

-- Adds to the addonTable:
--  InitializeConfig()
-- Uses from the addonTable:
--  ApplyLevelSettings()
--  ApplyDualWieldSettings()
--  SetUpdateInterval(seconds)

--------------------------
-- Variables -------------
--------------------------
local Addon = LibStub("AceAddon-3.0"):GetAddon(addonName)
local L = LibStub("AceLocale-3.0"):GetLocale(addonName, false)
local db

--------------------------
-- Upvalues --------------
--------------------------
local ApplyLevelSettings = addonTable.ApplyLevelSettings
local ApplyDualWieldSettings = addonTable.ApplyDualWieldSettings
local SetUpdateInterval = addonTable.SetUpdateInterval

--------------------------
-- Constants -------------
--------------------------
local defaults = {
	char = {
		enabled = true,
		dualwield = false
	},
	profile = {
		throttleinterval = 1.5,
		defaultlevel = 83,
		usetarget = true,
		bosslevel = 83
	}
}

local options = {
	name = L["TITLE"],
	type = "group",
	get = function(info) return db[info.option.arg][info[1]] end,
	args = {
		enabled = {
			type = "toggle",
			name = L["OPT_ENABLED"],
			desc = L["OPT_ENABLED_DESC"],
			arg = "char",
			set = function(_, value)
				if(value) then
					Addon:Enable()
				else
					Addon:Disable()
				end
			end,
			order = 0,
		},
		dualwield = {
			type = "toggle",
			name = L["OPT_DUALWIELD"],
			desc = L["OPT_DUALWIELD_DESC"],
			arg = "char",
			set = function(_, value)
				db.char.dualwield = value
				ApplyDualWieldSettings()
			end,
			order = 1,
		},
		throttleinterval = {
			type = "range",
			min = 0.1,
			max = 5.0,
			step = 0.35,
			name = L["OPT_THROTTLE"],
			desc = L["OPT_THROTTLE_DESC"],
			arg = "profile",
			set = function(_, value)
				db.profile.throttleinterval = value
				SetUpdateInterval(value)
			end,
			order = 2,
		},
		defaultlevel = {
			type = "range",
			min = 1,
			max = 83,
			step = 1,
			name = L["OPT_DEFAULTLEVEL"],
			desc = L["OPT_DEFAULTLEVEL_DESC"],
			arg = "profile",
			set = function(_, value)
				db.profile.defaultlevel = value
				ApplyLevelSettings()
			end,
			order = 3,
		},
		usetarget = {
			type = "toggle",
			name = L["OPT_USETARGET"],
			desc = L["OPT_USETARGET_DESC"],
			arg = "profile",
			set = function(_, value)
				db.profile.usetarget = value
				ApplyLevelSettings()
			end,
			order = 4,
		},
		bosslevel = {
			type = "range",
			min = 1,
			max = 83,
			step = 1,
			name = L["OPT_BOSSLEVEL"],
			desc = L["OPT_BOSSLEVEL_DESC"],
			arg = "profile",
			set = function(_, value)
				db.profile.bosslevel = value
				ApplyLevelSettings()
			end,
			disabled = function() return not db.profile.usetarget end,
			order = 5,
		},
	}
}

--------------------------
-- Functions -------------
--------------------------
local function RefreshConfig()
	ApplyLevelSettings()
	SetUpdateInterval(db.profile.throttleinterval)
end

function addonTable.InitializeConfig()
	-- Database setup
	db = LibStub("AceDB-3.0"):New(addonName.."DB", defaults)
	db.RegisterCallback(addonTable, "OnProfileChanged", RefreshConfig)
	db.RegisterCallback(addonTable, "OnProfileCopied", RefreshConfig)
	db.RegisterCallback(addonTable, "OnProfileReset", RefreshConfig)
	RefreshConfig()
	
	-- Options setup
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)
	options.args.profiles.dialogHidden = true
	LibStub("AceConfig-3.0"):RegisterOptionsTable(addonName, options)
	
	-- Options GUI
	local ACD = LibStub("AceConfigDialog-3.0")
	ACD:AddToBlizOptions(addonName, L["TITLE"])
	ACD:AddToBlizOptions(addonName, L["PROFILES"], L["TITLE"], "profiles")
	LibStub("LibAboutPanel").new(L["TITLE"], addonName)
	
	return db
end
